'''
Duplicate (most of) the functionality of perl Ffile.pm

In other words, read in a Fortran file, assembling the
line objects.
Provide mappers, rewriters, string conversion facilities
'''
from cStringIO import StringIO
from fortLine  import a_line
from assembler import vgen
from buf_iter  import buf_iter

def _ident(s):
    return [s]

class Ffile(object):
    def __init__(self,fobj):
        self.lines = vgen(a_line,buf_iter(fobj))
        self.fobj  = fobj

    @staticmethod
    def file(name):
        return Ffile(open(name))

    @staticmethod
    def here(str):
        return Ffile(StringIO(str))

    def str(self):
        '''return all of the original file lines concatenated together
        WARNING: do not use on large files !!
        '''
        return ''.join([l.rawline for l in self.lines])

    def readlines(self):
        '''Make Ffile function like readlines for regular files'''
        return [ ll for l in self.lines for ll in l.rawline.splitlines(True) ]

    def iterlines(self):
        '''instead of the whole line list, return an iterator
        that yields 1 rawline at a time
        '''
        return ( (ll for l in self.lines for ll in l.rawline.splitlines(True)) )

    def printit(self,out=None):
        if out:
            for l in self.iterlines():
                print >> out,l,
        else:
            for l in self.iterlines():
                print l,
    
    def write(self,fname):
        ff = open(fname,'w')

        self.printit(ff)

        ff.close()

    def map(self,lexi):
        for (cls,meth) in lexi:
            cls.map = meth
        for l in self.lines:
            for ll in l.map():
                yield ll
        for (cls,meth) in lexi:
            cls.map = _ident
